import 'package:equatable/equatable.dart';

/// Version of the touch events format expected by the native API
const int version = 1;

/// Types of the different gesture supported by the Zoning feature
enum ContentsquareGesture {
  /// tap - 6
  tap,

  /// longsPress - 8
  longPress,

  /// drag - 9
  drag,

  /// flick - 10
  flick,
}

/// Extension to map the int representation of a [ContentsquareGesture]
extension ContentsquareGestureInt on ContentsquareGesture {
  /// Return the int representation for the [ContentsquareGesture]
  int toInt() {
    switch (this) {
      case ContentsquareGesture.tap:
        return 6;
      case ContentsquareGesture.longPress:
        return 8;
      case ContentsquareGesture.drag:
        return 9;
      case ContentsquareGesture.flick:
        return 10;
    }
  }
}

/// Direction of a CSGesture
/// this concerns the Swipe (Drag/Flick) : UP, DOWN, LEFT, RIGHT
enum ContentsquareGestureDirection {
  /// up - 1
  up,

  /// down - 2
  down,

  /// left - 3
  left,

  /// right - 4
  right,
}

/// Extension to map the int representation of a [ContentsquareGestureDirection]
extension ContentsquareGestureDirectionInt on ContentsquareGestureDirection {
  /// Return the int representation for the [ContentsquareGestureDirection]
  int toInt() {
    switch (this) {
      case ContentsquareGestureDirection.up:
        return 1;
      case ContentsquareGestureDirection.down:
        return 2;
      case ContentsquareGestureDirection.left:
        return 3;
      case ContentsquareGestureDirection.right:
        return 4;
    }
  }
}

/// Struct to hold gesture related data
class ContentsquareGestureData extends Equatable {
  /// int - dragged/scrolled distance
  final int distance;

  /// CSGestureDirection (mapped to int) - finger direction - [1,2,3,4] -> [up, down, left, right] (see [ContentsquareGesture])
  final ContentsquareGestureDirection direction;

  /// int - velocity while scrolling
  final int velocity;

  /// Constructor for the CSGestureData data struct
  const ContentsquareGestureData({
    required this.distance,
    required this.direction,
    required this.velocity,
  });

  @override
  List<Object> get props => [distance, direction, velocity];
}

/// Struct to hold data for the touch events
class TouchEventData extends Equatable {
  /// String - target path inside the flutter widgets hierarchy
  final String path;

  /// CSGestureData - optional gesture related data
  final ContentsquareGestureData? gestureData;

  /// Constructor for the TouchEventData data struct
  const TouchEventData({
    required this.path,
    this.gestureData,
  });

  /// Cast TouchEventData to Map<String, dynamic> for platform channel serialization
  Map<String, dynamic> toJson() {
    return {
      'path': path,
      'distance': gestureData?.distance,
      'direction': gestureData?.direction.toInt(),
      'velocity': gestureData?.velocity,
    };
  }

  @override
  List<Object?> get props => [path, gestureData];
}
